""" $Id$

"""

__version__ = "$Revision$"
__author__ = "R. Le Gac"


from gluon.storage import Storage
from modifier import Modifier


FORM_MODIFIERS = 'form_modifiers'


def configure_forms(db, **extjs):
    """Helper functions to apply to all grid widgets a set of 
    configuration options. Useful to set plugins for example.
    
    The keyword arguments contain the configuration options for
    the Ext.form.FormPanel.
    
    """
    for tablename in db.tables:
        fm = FormModifier(tablename)
        fm.configure(**extjs)



class FormModifier(Modifier):
    """Helper tool to customize the form widget associated to a table.
    
    """
    
    def __init__(self, tablename):
        """Initialize the modifier persistent data.
        
        """
        Modifier.__init__(self, FORM_MODIFIERS, tablename)

        self.data.field_sets = []
        self.data.hidden_fields = []
        self.data.mapper = None
        

    def merge_fields(self, *fields, **kwargs):
        """Merge fields in a field set corresponding to 
        an Ext.form.FieldSet widget. Each field is identified by its 
        database field name.
        
        The keyword argument ExtJS contains the configuration options of
        the Ext.form.FieldSet widget.
        
        The keyword argument spacer add an Ext.Spacer widget on the bottom
        of the field set. Its value defined its height in pixels.
        
        """
        # by default the field occupies all the field set space
        extjs = {'defaults': {'anchor': '100%'}}

        if 'spacer' in kwargs:
            di = {'xtype': 'spacer', 'height': kwargs['spacer']}
            if 'items' in extjs:
                items.append(di)
            else:
                extjs['items'] = [di]

        if 'extjs' in kwargs:
            extjs.update(kwargs['extjs'])
        
        di = Storage(fields=fields, extjs= extjs)
        self.data.field_sets.append(di)


    def hide_fields(self, *fields):
        """Hide the list of fields in the form.
        Each field is identified by its database field name.
        
        """
        for el in fields:
            if el not in self.data.hidden_fields:
                self.data.hidden_fields.append(el)
    
    def set_mapper(self, func):
        """Link the mapper tool to the function func.
        
        A mapper tool is related to the organization of the field/fieldest
        in the form. By default the form layout is a column where fields are 
        pushed from the top to the bottom. By using the layout configuration 
        option it is possible to organize the field/fieldset in horizontal
        box, vertical box, columns or in table.
        
        This approach does not allow to map field/fieldset in panels
        contains in the form, like an Ext.TabPanel. The mapper tool allows to 
        handle this case. It is a function with one arguments, the list of 
        field/fieldset. The function return a dictionary with the structure of
        panels embedded in the form an the fields they contained.

        Example of mapper can be found in the mapper module.
        
        """
        self.data.mapper = func