""" $Id$

"""

__version__ = "$Revision$"
__author__ = "R. Le Gac"


from gluon.storage import Storage
from gluon.tools import PluginManager


class Modifier(object):
    """Base class to construct Modifier tools.
    
    """

    def __init__(self, dpname, tablename=None):
        """Link the modifier with the persistent data/parameters 
        of the plugin. Persistent data are identified by their name dpname.
        
        Persistent data are initialized if they don't exist.
        As a Storage id tablename is not specified, as a dict otherwise.
        The key tablename is add to the dictionary and point to a Storage.
        
        In all case, the storage contains the key extjs used to store
        configuration option of the associate ExtJS widget.

        The class attribute data is a reference to the Storage.

        """    
        p = PluginManager('dbui')
        
        if tablename:        
            if dpname not in p.dbui:
                p.dbui[dpname] = {}

            if tablename not in p.dbui[dpname]:
                p.dbui[dpname][tablename] = Storage(extjs={}) 
                
            self.data = p.dbui[dpname][tablename]
            
        else:
            if dpname not in p.dbui:
                p.dbui[dpname] = Storage(extjs={})
                
            self.data = p.dbui[dpname]
 
        
    def append_plugins(self, *plugins):
        """Add a list of plugins to the associated widget
        Ext.form.FormPanel, Ext.grid.GridPanel, ....
        
        A plugin is defined by a string containing is pType
        or by a dictionary containing the pType and the plugin
        configuration options, i.e. {'ptype': foo, 'option1': faa,...}
        
        For more information on ptype sSee ExtJS documentation.
        
        """
        if 'plugins' in self.data.extjs:
            self.data.extjs['plugins'].append(plugins)
            
        else:
            self.data.extjs['plugins'] = plugins


    def configure(self, **extjs):
        """Update the configuration options of the associated widget:
        Ext.form.FormPanel, Ext.grid.GridPanel, ....
        
        Existing value of the configuration options are replace by those 
        defined by the keyword arguments.
        
        If the keyword argument plugins is used, the current list of plugins
        are replaced by the new one. To append plugins to the existing list
        uses the method append_plugins.
        
        If the keyword argument xtype is used the widget associated by default
        is replaced by a new widget. Will work only if the new widget will 
        inherit from the default one. See the documentation of 
        Ext.Component to learn how to manipulate xtype configuration option.
        
        """
        self.data.extjs.update(extjs)

