""" $Id$
    Author: R. Le Gac
"""

__version__ = "$Revision$"


from gluon.validators import IS_IN_SET

class SetField(dict):
    """ dictionary mapping the set fields: 
    SetField[table][field] = list of values
    
    """ 

    def __init__(self, db):
        """ Scan the database model to identify set field.
        A field with predefined values.
        
        """

        for table in db.tables:
            fields = [db[table][el] for el in db[table].fields]
            
            for field in fields:
                #
                # SET FIELD
                # is defined by the validator IS_IN_SET.
                #
                validators = field.requires

                if not isinstance(validators, (list, tuple)):
                    validators = [field.requires]
            
                for vdt in validators:
                    if isinstance(vdt, IS_IN_SET):
                        
                        if table not in self:
                            self[table] = {}
                            
                        self[table][field.name] = vdt.theset


    def is_set_field(self, table, field):
        """ Return true is the table.field is a set field.
        
        """
        if table in self:
            return field in self[table]
        
        else:
            return False
        
         
    def get_set_data(self, table, field):
        """ Return a list with the set data.
        
        """
        if self.is_set_field(table, field):
            return self[table][field]
        
        else:
            return None
