/**
 * The paging bottom bar plugin
 * The ptype of this component is pGridPaging.
 *  
 *  
 * NOTE: the number of row load in the grid at the first time
 * is defined by the grid when loading the store.
 * 
 * @version $Id$
 * 
 */
Ext.namespace('App.grid');

App.grid.Paging = Ext.extend(Object, {

    /**
     * Private parameter identifying the type of plugin
     */
    ptype: 'pGridPaging',

    /**
     * Plugin initialization
     */
    init: function (grid) {

        var bbar;
        
        // link the bbar and the grid store
        bbar = grid.getBottomToolbar();
        bbar.bindStore(grid.store);

        // add the slider and the export button
        bbar.add('-', 
                 'Rows per page', 
                 {
                    xtype: 'slider',
                    plugins: new Ext.slider.Tip(),
                    listeners: {
                        changecomplete: this.onChangePageSize,
                        scope: bbar
                    },
                    minValue: 1,
                    width: 100
                 }, 
                 '->',
                 {
                    xtype: 'xbuttondownload',
                    text: 'Export to CSV',
                    url: App.csvUrl + '?tableName=' + App.getTableName(grid.store)
                 }
        );
        
        bbar.show();
        
        // Initialize the parameters of the paging and slider widgets.
        // Depends on the status of the store.
        // Initialization is only performed once.
        if (grid.store.getTotalCount() > 0) {
            
            this.onInit.call(grid, grid.store);
        
        } else {
        
            grid.store.on('load', this.onInit, grid, {single: true});
        
        }
        
        // update the slider parameter after a write action
        // which destroy and create records
        grid.store.on('write', this.onWrite, grid);
    },
    
    /**
     * Handler to change the number of rows per page
     * the scope is the bottom tool bar
     */
    onChangePageSize: function (slider, newValue, thumb) {
        var bbar = this;
        bbar.pageSize = newValue;
        bbar.moveFirst();
    },
    
    /**
     * Handler to initialize the number of rows per page and the number of page
     * the scope is the grid
     */
    onInit: function (store, records, options) {

        var bbar, 
            grid = this, 
            nRows = store.getCount(), 
            slider;
                
        bbar = grid.getBottomToolbar();
        bbar.pageSize = nRows;

        slider = bbar.findByType('slider')[0];
        slider.setMaxValue(store.getTotalCount());
        slider.setValue(nRows);
        
    },
    
    /**
     * Handler to update the slider parameter after a store write action.
     * The latter create or destroy records in the store.
     * The scope is the bottom grid
     */
    onWrite: function () {
        
        var bbar,
            grid = this,
            slider;

        bbar = grid.getBottomToolbar();
        
        slider = bbar.findByType('slider')[0];
        slider.setMaxValue(grid.store.getTotalCount());
        slider.setValue(grid.store.getCount());
        
    }
});

Ext.preg('pGridPaging', App.grid.Paging);
