/**
 * The row editor context menu plugin
 *
 * Display the context menu when the user right click on a row.
 * The content of the menu allows to manipulate the App.grid.RowEditor
 * 
 * Requires the App.grid.GridRowEditor plugin.
 * 
 * The ptype of this component is pGridRowEditorContextMenu.
 * 
 * @extends Object
 * @version $Id$
 * 
 */

Ext.namespace('App.grid');

App.grid.RowEditorContextMenu = Ext.extend(Object, {

    /**
     * Private attribute identifying the type of plugin
     */
    ptype: 'pGridRowEditorContextMenu',
    
    /**
     * Plugin initialization
     * @param {Object} grid
     */
    init: function (grid) {
                
        var menu = new Ext.menu.Menu();
        
        // protection		
        if (!grid.rowEditor) {
            throw new Error('no grid row editor !!!');
        }

        grid.addListener('containercontextmenu', this.onContainerContextMenu, menu);
        grid.addListener('headercontextmenu', this.onHeaderContextMenu, menu);
        grid.addListener('rowcontextmenu', this.onRowContextMenu, menu);
        
        menu.add({
            text: 'Add',
            iconCls: 'silk-add',
            handler: grid.rowEditor.onAddRow,
            scope: grid.rowEditor
        }, '-', {
            text: 'Delete',
            iconCls: 'silk-delete',
            handler: grid.rowEditor.onDeleteRow,
            scope: grid.rowEditor
        }, '-', {
            text: 'Duplicate',
            iconCls: 'silk-clone',
            handler: grid.rowEditor.onDuplicateRow,
            scope: grid.rowEditor
        }, {
            text: 'Update',
            iconCls: 'silk-update',
            handler: grid.rowEditor.onEditRow,
            scope: grid.rowEditor
        }, '-', {
            text: 'View',
            iconCls: 'silk-view',
            handler: grid.rowEditor.onViewRow,
            scope: grid.rowEditor
        });
    },
    
    /**
     * Prvate handler displaying the row context menu in an empty grid
     */
    onContainerContextMenu: function (grid, event) {
        var menu = this;
        
        event.stopEvent();
        menu.showAt(event.getXY());
        
    },
    /**
     * Private handler displaying the context menu in the header
     * Nothing is display in that case.
     */
    onHeaderContextMenu: function (grid, columIndex, event) {
        event.stopEvent();
    },
    /**
     * Private handler displaying the row context menu
     * The scope use by the handler is the menu object.
     */
    onRowContextMenu: function (grid, rowIndex, event) {
        
        var menu = this;
        
        event.stopEvent();
        grid.selModel.selectRow(rowIndex);		
        menu.showAt(event.getXY());

    }

});

Ext.preg('pGridRowEditorContextMenu', App.grid.RowEditorContextMenu);
