/**
 * appmgr.js
 * 
 * The application manager class.
 * 
 * The class App.Mgr configure the application and launch it.
 * It is design for database web application.
 * 
 * The layout is a tab widget with a tree panel encapsulated in a viewport.
 * The tree exposes two main nodes Forms and Tables. The children of these
 * nodes are the tables of the database.
 *  
 * The class uses the cfgSvc web service to retrieve the configuration 
 * options the viewport and latter on of the grid and form widgets.
 * 
 * $Id$
 * 
 */
Ext.namespace('App');

App.Mgr = Ext.extend(Object, {
    
    /**
     * constructor of the application manager
     * @param {Object} config
     */
    constructor: function (config) {
        Ext.apply(this, config);
        App.Mgr.superclass.constructor.apply(this);
        
        // If the MathJax library is required load it
        if (App.withMathJax) {
            App.loadMathJax(this.configure, this);
        } else {
            this.configure();
        }
    },
    
    /**
     * Callback function to get the viewport configuration from the server
     */
    configure: function () {
        Dbui.getViewport(this.launch, this);		
    },
    
    /**
     * Call back function to launch the application.
     * 
     * This method is compatible with the Ext.Direct protocol
     * http://www.extjs.com/products/extjs/direct.php
     * 
     * @param {Object} provider
     * @param {Object} response
     */
    launch: function (provider, response) {
        
        var cfg, 
            vpCfg, 
            treeNode, 
            viewport;

        // The treePanel contains a root node.        
        // The name of the root node of the tree is App.name
        // It is not visible.

        treeNode = new Ext.tree.TreeNode({
            id: App.name,
            text: App.name
        });
        
        // Define the application interface:
        // a viewport with a tree panel and a tab panel.
        // The tree panel is referenced as treePanel 
        // and the tab panel as tabPanel.
        // These panels can therefore be accessed via 
        // the property this.treePanel ....

        vpCfg = {
            layout: 'border',
            title: 'Ext Layout Browser',
            items: [{
                region: 'west',
                title: App.name,
                width: 200,
                collapsible: true,
                autoScroll: true,
                split: true,
                xtype: 'treepanel',
                rootVisible: false,
                root: treeNode,
                ref: 'treePanel'
            }, {
                region: 'center',
                xtype: 'tabpanel',
                ref: 'tabPanel',
                autoScroll: true,
                defaults: {layout: 'fit'}
            }]
        };

        // get the configuration options return by the server
        cfg = response.result;
        
        // Instanciate the viewport
        vpCfg = Ext.apply(vpCfg, cfg);
        viewport = new Ext.Viewport(vpCfg);
        
        // Disable contextmenu in the treepanel
        viewport.treePanel.on('contextmenu', function (n, e) {
            e.stopEvent();
        });
    }
});
