/**
 * Plugin InTreeGrid.
 * 
 * The App.Viewport contains a tree and a tab panels.
 * This plugin add a topNode in the tree which contains child.
 * When a child is clicked a new tab appears in the tab panel.
 * 
 * The name of the child nodes are defined by the property 'nodes'.
 * For example the name of the tables for a database application.
 * 
 * The tab contains a App.grid.GridPanel panel. 
 * Its configuration is retrieve from the server.
 * the property wdgCfg allow to add additional configuration property
 * to the grid like plugins.
 *
 * The ptype of this component is pInTreeGrid.
 * 
 * MANDATORY PROPERTIES:
 *      - models
 *      - topNodeName
 *      - nodes
 * 
 * @author legac
 * @extend App.viewport.InTreePanelBase
 * @version $Id$
 * 
 */
Ext.namespace('App.viewport');

App.viewport.InTreePanelGrid = Ext.extend(App.viewport.InTreePanelBase, {

    /**
     * private function to instantiate the grid and to add it to the tab
     * 
     * @param {Object} container
     * @param {Object} component
     * @param {Object} index
     */
    addWdg: function (container, component, index) {

        var cfg, 
            grid, 
            gridfilter, 
            table, 
            wdg;
        
        function createGrid(provider, response) {
            cfg = response.result;
            wdg = Ext.ComponentMgr.create(cfg);
            
            if (cfg.hasOwnProperty('filterModel')) {
                // instantiate the grid and the filter
                grid = wdg;
                gridfilter = Ext.ComponentMgr.create(cfg.filterModel);
                gridfilter.bind(grid);
                
                // embedded the grid and the filter in a panel
                wdg = new Ext.Panel({
                    layout: 'border',
                    border: false,
                    items: [{
                        region: 'center',
                        layout: 'fit',
                        items: [grid]
                    }, {
                        region: 'east',
                        layout: 'form',
                        collapsible: true,
                        collapsed: true,
                        defaults: {anchor: '100%'},
                        frame: true,
                        items: [gridfilter],
                        width: 300,
                        split: true,
                        buttons: [{text: 'reset', ref: '../../resetButton'}]
                    }]
                });

                // connect the reset button
                wdg.resetButton.on('click', gridfilter.onReset, gridfilter);
            }
                    
            component.add(wdg);
            component.doLayout();
        }
        
        // protection -- add widget when we are concern !				
        if (this.getTopNodeNameFromId(component.id) !== this.topNodeName) {
            return;
        }
        
        // get the name of the database table from the tab id
        // NOTE: this implementation assume that the node name is equal to
        // the database taable name
        table = this.getNodeNameFromId(component.id);
        
        // Ask the server the widget configuration and create it
        Dbui.getGrid(table, createGrid, this);
    }
});

Ext.preg('pInTreeGrid', App.viewport.InTreePanelGrid);